package org.kantega.jexmec;

/*
 * Copyright 2011 Kantega AS
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

import java.util.Collection;

/**
 * Keeps track of plugins currently installed in the application. Application and plugin
 * code should call the getPlugins() method when they need to iterate over installed plugins.
 */
public interface PluginManager<P> {
    /**
     * Return a list of the plugins currently installed in this PluginManager
     * @return  the list of plugins
     */
    Collection<P> getPlugins();

    /**
     * Returns the subset of plugins assignable from the given type <code>T</code>
     * @param pluginClass the plugin class to filter for
     * @param <T> type of plugin class
     * @return a filtered list of plugins implementing <code>pluginClass</code>
     */
    <T extends P> Collection<T> getPlugins(Class<T> pluginClass);

    /**
     * Register a PluginManagerListener for event notification.
     * @param listener the listener to add
     */
    void addPluginManagerListener(PluginManagerListener<P> listener);

    /**
     * Unregister a PluginManagerListener for event notification.
     * @param listener the listener to remove
     */
    void removePluginManagerListener(PluginManagerListener<P> listener);

    /**
     * Return the ClassLoader that initiated the loading of the given plugin. Note that this
     * need not be the same as <code>plugin.getClassLoader()</code>
     * @param plugin the plugin for which to return the loading plugin
     * @return the initiating ClassLoader
     */
    ClassLoader getClassLoader(P plugin);
}
