package org.kantega.jexmec;

/*
 * Copyright 2011 Kantega AS
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

import java.util.Collection;

/**
 * Listener class for {@link PluginManager} life cycle events.
 */
public abstract class PluginManagerListener<P> {

    /**
     * Fired when the plugin manager is about to be started
     * @param pluginManager the plugin manager event
     */
    public void beforePluginManagerStarted(PluginManager<P> pluginManager) {}

    /**
     * Fired after the plugin manager has started.
     * @param pluginManager the plugin manager event
     */
    public void afterPluginManagerStarted(PluginManager pluginManager) {}

    /**
     * Fired when the plugin manager is about to be stopped
     * @param pluginManager the plugin manager event
     */
    public void beforePluginManagerStopped(PluginManager<P> pluginManager) {}

    /**
     * Fired when the plugin manager has been stopped
     * @param pluginManager the plugin manager event
     */
    public void afterPluginManagerStopped(PluginManager pluginManager) {}

    /**
     * Fired when a plugin class loader provider requests a class loader to be added
     *
     */
    public void beforeClassLoaderAdded(PluginManager<P> pluginManager,
                                       ClassLoaderProvider classLoaderProvider,
                                       ClassLoader classLoader) {}

    /**
     * Fired when the plugin manager has finished adding a class loader
     *
     */
    public void afterClassLoaderAdded(PluginManager<P> pluginManager,
                                       ClassLoaderProvider classLoaderProvider,
                                       ClassLoader classLoader) {}

    /**
     * Fired when a plugin class loader provider requests a class loader to be removed
     *
     */
    public void beforeClassLoaderRemoved(PluginManager<P> pluginManager,
                                       ClassLoaderProvider classLoaderProvider,
                                       ClassLoader classLoader) {}

    /**
     * Fired when the plugin manager has finished adding a class loader
     *
     */
    public void afterClassLoaderRemoved(PluginManager<P> pluginManager,
                                       ClassLoaderProvider classLoaderProvider,
                                       ClassLoader classLoader) {}


    /**
     * Fired just before the plugins are activated (made available in the registry)
     * In this state, plugins are not yet visible to {@link PluginManager#getPlugins()}
     */
    public void beforeActivation(PluginManager<P> pluginManager,
                                       ClassLoaderProvider classLoaderProvider,
                                       ClassLoader classLoader,
                                       PluginLoader<P> pluginLoader,
                                       Collection<P> plugins) {}

    /**
     * Fired when when plugins have been activated (made available in the registry). At this
     * point the plugins are visible to {@link PluginManager#getPlugins()}.
     */
    public void afterActivation(PluginManager<P> pluginManager,
                                       ClassLoaderProvider classLoaderProvider,
                                       ClassLoader classLoader,
                                       PluginLoader<P> pluginLoader,
                                       Collection<P> plugins) {}


    /**
     * Fired when a collection of plugins is about to be passivated. At this point they have not been removed from the plugin
     * registry, and are visible to {@link PluginManager#getPlugins()}
     */
    public void beforePassivation(PluginManager<P> pluginManager,
                                       ClassLoaderProvider classLoaderProvider,
                                       ClassLoader classLoader,
                                       PluginLoader<P> pluginLoader,
                                       Collection<P> plugins) {}

    /**
     * Fired when a collection of plugins have been passivated. That is they have been removed from the
     * plugin registry, but are not yet unloaded by their plugin loaders.
     */
    public void afterPassivation(PluginManager<P> pluginManager,
                                       ClassLoaderProvider classLoaderProvider,
                                       ClassLoader classLoader,
                                       PluginLoader<P> pluginLoader,
                                       Collection<P> plugins) {}


    /**
     * Fired when the list of plugins has changed because plugins were added or removed.
     * @param plugins the updated list of plugins
     */
    public void pluginsUpdated(Collection<P> plugins) {}

    /**
     * Fired when a PluginLoader has thrown an exception during plugin loading
     *
     */
    public void pluginLoadingFailedWithException(PluginManager<P> pluginManager,
                                       ClassLoaderProvider classLoaderProvider,
                                       ClassLoader classLoader,
                                       PluginLoader<P> pluginLoader,
                                       Throwable exception) {}
}


